@@ -3,7 +3,7 @@
 modelr.web.server
 ===================
 
-Main program to strat a web server.
+Main program to start a web server.
 
 Created on Apr 30, 2012
 
@@ -20,6 +20,9 @@ from modelr.web.urlargparse import SendHelp, ArgumentError, \
      URLArgumentParser
 import traceback
 import json
+import multiprocessing as mp
+import ssl
+
 
 class MyHandler(BaseHTTPRequestHandler):
     '''
@@ -96,13 +99,15 @@ class MyHandler(BaseHTTPRequestHandler):
 
                 self.send_response(200)
                 self.send_header('Access-Control-Allow-Origin', '*')
-                self.send_header('Access-Control-Allow-Headers', 'X-Request, X-Requested-With')
+                self.send_header('Access-Control-Allow-Headers',
+                                 'X-Request, X-Requested-With')
                 self.send_header('Content-type', 'application/json')
                 self.end_headers()
                 
                 script_main = namespace['run_script']
                 add_arguments = namespace['add_arguments']
-                short_description = namespace.get('short_description', 'No description')
+                short_description = namespace.get('short_description',
+                                                  'No description')
 
                 parser = URLArgumentParser(short_description)
                 add_arguments(parser)
@@ -114,7 +119,8 @@ class MyHandler(BaseHTTPRequestHandler):
             if uri.path == '/available_scripts.json':
                 self.send_response(200)
                 self.send_header('Access-Control-Allow-Origin', '*')
-                self.send_header('Access-Control-Allow-Headers', 'X-Request, X-Requested-With')
+                self.send_header('Access-Control-Allow-Headers',
+                                 'X-Request, X-Requested-With')
                 self.send_header('Content-type', 'application/json')
                 self.end_headers()
                 
@@ -138,10 +144,18 @@ class MyHandler(BaseHTTPRequestHandler):
                 
             script_main = namespace['run_script']
             add_arguments = namespace['add_arguments']
-            short_description = namespace.get('short_description', 'No description')
+            short_description = namespace.get('short_description',
+                                              'No description')
             
             print "parameters", parameters
-            self.run_script(script[0], script_main, add_arguments, short_description, parameters)
+            p = mp.Process(
+                target=self.run_script, args=(script[0],script_main,
+                                              add_arguments,
+                                              short_description,
+                                              parameters))
+            p.start()
+            p.join()
+           
             
         except Exception as err:
             self.send_response(400)
@@ -159,7 +173,8 @@ class MyHandler(BaseHTTPRequestHandler):
             
             raise
         
-    def run_script(self, script, script_main, add_arguments, short_description, parameters):
+    def run_script(self, script, script_main, add_arguments,
+                   short_description, parameters):
         '''
         Run a script 
         
@@ -177,10 +192,12 @@ class MyHandler(BaseHTTPRequestHandler):
             self.send_header('Content-type', 'text/html')
             self.end_headers()
             
-            template = self.server.jenv.get_template('ScriptHelp.html')
+            template = \
+              self.server.jenv.get_template('ScriptHelp.html')
             
             print parameters
-            html = template.render(script=script, parser=parser, parameters=parameters)
+            html = template.render(script=script, parser=parser,
+                                   parameters=parameters)
             self.wfile.write(html)
             return
         except ArgumentError as err:
@@ -188,7 +205,8 @@ class MyHandler(BaseHTTPRequestHandler):
             self.send_header('Content-type', 'text/html')
             self.end_headers()
                 
-            self.wfile.write('<p><b>Error:</b> %s</p>' % (err.args[0],))
+            self.wfile.write('<p><b>Error:</b> %s</p>'
+                             % (err.args[0],))
             self.wfile.write(parser.help_html)
             return
         
@@ -199,7 +217,9 @@ class MyHandler(BaseHTTPRequestHandler):
         self.end_headers()
             
         self.wfile.write(jpeg_data)
-    
+
+        del jpeg_data
+        
     def get_available_scripts(self):
         '''
         Returns a list of all the scripts in the scripts directory.
@@ -217,7 +237,8 @@ class MyHandler(BaseHTTPRequestHandler):
                 namespace = {}
                 with open(join(scripts_dir, script), 'r') as fd:
                     exec fd.read() in namespace
-                short_doc = namespace.get('short_description', 'No doc')
+                short_doc = namespace.get('short_description',
+                                          'No doc')
                 print script, namespace
                 available_scripts.append((script, short_doc))
             except Exception, e:
@@ -238,13 +259,22 @@ class MyHandler(BaseHTTPRequestHandler):
         self.send_header('Content-type', 'text/html')
         self.end_headers()
         
-        html = template.render(msg=msg, available_scripts=self.get_available_scripts())
+        html = \
+          template.render(msg=msg,
+                    available_scripts=self.get_available_scripts())
         self.wfile.write(html)
         
         
     def do_POST(self):
-        self.send_error(404, 'Post request not supportd yet: %s' % self.path)
+        self.send_error(404, 'Post request not supportd yet: %s'
+                        % self.path)
 
+# Locations of the PEM files for SSL
+# If this doesn't work, an alternative would be to store the
+# full chain including the private key, as described here:
+# http://www.digicert.com/ssl-support/pem-ssl-creation.htm
+CERTFILE = '/etc/ssl/modelr/public.pem'
+KEYFILE = '/etc/ssl/private/private.pem'
 
 def main():
     '''
@@ -260,7 +290,19 @@ def main():
         server = HTTPServer((args.host, args.port), MyHandler)
         server.jenv = Environment(loader=PackageLoader('modelr',
                                                     'web/templates'))
-        
+        # This provides SSL, serving over HTTPS.
+        # This approach will not allow service over HTTP.
+        # I think we should allow both, since there is no
+        # real reason for modelr-server to be secure.
+        # I don't know if we need to check the certificate
+        # on the client side too, or if doing it this way
+        # will satisfy the browser and that's enough.
+        server.socket = ssl.wrap_socket(server.socket,
+                                         certfile=CERTFILE,
+                                         keyfile=KEYFILE,
+                                         server_side=True
+                                         )
+                                        
         print 'started httpserver...'
         server.serve_forever()
     except KeyboardInterrupt:
